
create table "REPC_CONFIG"
(
 "Id" int not null primary key,
 "Exclusive" smallint,
 "Version" numeric(4,1) not null,
 "CodePage" int
);
/*--------------------------------------------------------------*/

create table "REPC_PROJECTS"
(
 "Id" int not null primary key,
 "Name" varchar(63) not null unique collate unicode,
 "AccessMethod" smallint not null,

 check("AccessMethod" in (1,2,3,4))
);
/*--------------------------------------------------------------*/

create table "REPC_DATABASES"
(
 "Id" int not null primary key,
 "Id_Project" int not null references "REPC_PROJECTS" on update cascade on delete no action,
 "Name" varchar(63) not null collate unicode,
 "Conn_string" varchar(255) not null,
 "CharSet" varchar(31),
 "AutoLogin" smallint
);

create unique index "REPC_IDX_DATABASES" on "REPC_DATABASES" ("Name", "Id_Project");
/*--------------------------------------------------------------*/

create table "REPC_DBUSERS"
(
 "Id" int not null primary key,
 "Id_Database" int not null references "REPC_DATABASES" on update cascade on delete cascade,
 "UserName" varchar(31) not null,
 "DefLogin" varchar(31),
 "DefRole" varchar(31)
);

create unique index "REPC_IDX_DBUSERS" on "REPC_DBUSERS" ("UserName", "Id_Database");
/*--------------------------------------------------------------*/

create table "REPC_GROUPS"
(
 "Id" int not null primary key,
 "Id_Project" int not null references "REPC_PROJECTS" on update cascade on delete no action,

 "Code" varchar(31) not null collate unicode,
 "Name" varchar(63) not null collate unicode
);

create unique index "REPC_IDX_GROUPS_1" on "REPC_GROUPS" ("Code", "Id_Project");
create unique index "REPC_IDX_GROUPS_2" on "REPC_GROUPS" ("Name", "Id_Project");
/*--------------------------------------------------------------*/

create table "REPC_REPORTS"
(
 "Id" int not null primary key,
 "Id_Group" int not null references "REPC_GROUPS" ("Id") on update cascade on delete no action,
 "Name" varchar(63) not null collate unicode,
 "Grant_public" smallint,

 "Desc_short" varchar(2000),
 "Desc_len" int,
 "Desc" blob sub_type text segment size 80,
 "Sfm" blob sub_type 0 segment size 80,
 "Unused" smallint,

 "CreateTime" timestamp,
 "ModTime" timestamp
);
/*--------------------------------------------------------------*/

create table "REPC_REPORT_PARAMS"
(
 "Id"	int not null primary key,
 "UserName" varchar(31) not null,
 "Id_Report" int not null references "REPC_REPORTS" ("Id") on update cascade on delete cascade,
 "Number" int not null,
 "Value" varchar(255),
 "ValueMemo" blob sub_type text segment size 80
);

create unique index "REPC_IDX_REPORT_PARAMS" on "REPC_REPORT_PARAMS" ("UserName", "Id_Report", "Number");
/*--------------------------------------------------------------*/

create table "REPC_DOCUMENTS"
(
 "Id"	int not null primary key,
 "Id_Database" int not null references "REPC_DATABASES" on update cascade on delete no action,
 "DocDate" date not null,

 "Name" varchar(63) not null collate unicode,
 "Desc" blob sub_type text segment size 80,
 "Sfr" blob sub_type 0 segment size 80,
 "Length" int,
 "Grant_public" smallint,

 "CreateUser" varchar(31),
 "CreateTime" timestamp
);

create unique index "REPC_IDX_DOCUMENTS_1" on "REPC_DOCUMENTS" ("Name", "Id_Database");
create index "REPC_IDX_DOCUMENTS_2" on "REPC_DOCUMENTS" ("DocDate", "Id");
/*--------------------------------------------------------------*/

create table "REPC_REPORT_GRANTS"
(
 "Id"	int not null primary key,
 "UserName" varchar(31) not null,
 "Id_Report" int not null references "REPC_REPORTS" ("Id") on update cascade on delete cascade
);

create unique index "REPC_IDX_REPORT_GRANTS" on "REPC_REPORT_GRANTS" ("UserName", "Id_Report");
/*--------------------------------------------------------------*/

create table "REPC_DOCUMENT_GRANTS"
(
 "Id"	int not null primary key,
 "UserName" varchar(31) not null,
 "Id_Document" int not null references "REPC_DOCUMENTS" ("Id") on update cascade on delete cascade
);

create unique index "REPC_IDX_DOCUMENT_GRANTS" on "REPC_DOCUMENT_GRANTS" ("UserName", "Id_Document");
/*--------------------------------------------------------------*/

create generator "REPC_PROJECTS_GEN";
create generator "REPC_DATABASES_GEN";
create generator "REPC_DBUSERS_GEN";
create generator "REPC_GROUPS_GEN";
create generator "REPC_REPORTS_GEN";
create generator "REPC_REPORT_PARAMS_GEN";
create generator "REPC_DOCUMENTS_GEN";
create generator "REPC_REPORT_GRANTS_GEN";
create generator "REPC_DOCUMENT_GRANTS_GEN";
/*--------------------------------------------------------------*/

create exception "REPC_EXC_CONFIG_1" 'You cannot delete from table REPC_CONFIG!';

create exception "REPC_EXC_PROJECTS_1" 'You cannot delete a Project that has databases!';
create exception "REPC_EXC_PROJECTS_2" 'You cannot delete a Project that has groups of reports!';

create exception "REPC_EXC_DATABASES_1" 'A database with this name already exists in this Project!';
create exception "REPC_EXC_DATABASES_2" 'You cannot delete a Database that has users!';
create exception "REPC_EXC_DATABASES_3" 'You cannot delete a Database that has documents!';

create exception "REPC_EXC_DBUSERS_1" 'This UserName already exists for this Database!';

create exception "REPC_EXC_GROUPS_1" 'A group with this name or code already exists in this Project!';
create exception "REPC_EXC_GROUPS_2" 'You cannot delete a Group that has reports!';

create exception "REPC_EXC_REPORTS_1" 'A report with this name already exists in this Project!';

create exception "REPC_EXC_DOCUMENTS_1" 'You cannot modify a document field: Id_Database!';
create exception "REPC_EXC_DOCUMENTS_2" 'You cannot modify a document field: Desc!';
create exception "REPC_EXC_DOCUMENTS_3" 'You cannot modify a document field: Sfr!';
create exception "REPC_EXC_DOCUMENTS_4" 'You cannot modify a document field: Length!';
create exception "REPC_EXC_DOCUMENTS_5" 'You cannot modify a document field: CreateUser!';
create exception "REPC_EXC_DOCUMENTS_6" 'You cannot modify a document field: CreateTime!';
create exception "REPC_EXC_DOCUMENTS_7" 'A document with this name already exists for this Database!';
create exception "REPC_EXC_DOCUMENTS_8" 'You cannot modify or drop a document created by another user!';

create exception "REPC_EXC_REPORT_GRANTS_1" 'You cannot update table REPC_REPORT_GRANTS!';

create exception "REPC_EXC_DOCUMENT_GRANTS_1" 'You cannot change grants for a document created by another user!';
create exception "REPC_EXC_DOCUMENT_GRANTS_2" 'You cannot update table REPC_DOCUMENT_GRANTS!';
/*--------------------------------------------------------------*/

set term ^;

/*--------------------------------------------------------------*/

create procedure "REPC_PROC_CMP"(in_s1 varchar(255), in_s2 varchar(255))
returns ("Cmp" int)
as
begin

if( in_s1 is null and in_s2 is null ) then begin "Cmp" = 0; exit; end
if( in_s1 is null ) then begin "Cmp" = -1; exit; end
if( in_s2 is null ) then begin "Cmp" = 1; exit; end
if( in_s1 = in_s2 ) then begin "Cmp" = 0; exit; end

if( in_s1 collate unicode > in_s2 collate unicode ) then "Cmp" = 1;
else if( in_s1 collate unicode < in_s2 collate unicode ) then "Cmp" = -1;
else "Cmp" = 0;

end ^
/*--------------------------------------------------------------*/

create procedure "REPC_PROC_COUNT_REPORTS"(in_group int, in_actual int, in_user varchar(32))
returns("Result" int)
as
begin
"Result" = null;
if( in_group is null ) then begin suspend; exit; end

if( in_actual is null ) then in_actual = 0;

if( in_actual != 0 and in_user is not null ) then
begin
  select count(*) from "REPC_REPORTS" r
  left join "REPC_REPORT_GRANTS" g on (g."Id_Report" = r."Id" and g."UserName" = :in_user)
  where r."Id_Group" = :in_group
  and (r."Unused" is null or r."Unused" = 0)
  and (g."Id" is not null or coalesce(r."Grant_public",0) != 0)
  into :"Result";
end
else if( in_actual != 0 ) then
begin
  select count(*) from "REPC_REPORTS" where "Id_Group" = :in_group
  and ("Unused" is null or "Unused" = 0)
  into :"Result";
end
else if( in_user is not null ) then
begin
  select count(*) from "REPC_REPORTS" r
  left join "REPC_REPORT_GRANTS" g on (g."Id_Report" = r."Id" and g."UserName" = :in_user)
  where r."Id_Group" = :in_group
  and (g."Id" is not null or coalesce(r."Grant_public",0) != 0)
  into :"Result";
end
else
begin
  select count(*) from "REPC_REPORTS" where "Id_Group" = :in_group into :"Result";
end

if( "Result" is not null and "Result" = 0 ) then "Result" = null;
suspend;
end ^
/*--------------------------------------------------------------*/

create procedure "REPC_PROC_COUNT_DB"(in_project int)
returns("Result" int)
as
begin
"Result" = null;
if( in_project is null ) then begin suspend; exit; end
select count(*) from "REPC_DATABASES" where "Id_Project" = :in_project into :"Result";
if( "Result" is not null and "Result" = 0 ) then "Result" = null;
suspend;
end ^
/*--------------------------------------------------------------*/

create procedure "REPC_PROC_COUNT_DBUSERS"(in_db int)
returns("Result" int)
as
begin
"Result" = null;
if( in_db is null ) then begin suspend; exit; end
select count(*) from "REPC_DBUSERS" where "Id_Database" = :in_db into :"Result";
if( "Result" is not null and "Result" = 0 ) then "Result" = null;
suspend;
end ^
/*--------------------------------------------------------------*/

create procedure "REPC_PROC_COUNT_PRUSERS"(in_project int)
returns("Result" int)
as
begin
"Result" = null;
if( in_project is null ) then begin suspend; exit; end
select count(*) from "REPC_DBUSERS" g join "REPC_DATABASES" d on (d."Id" = g."Id_Database")
where d."Id_Project" = :in_project into :"Result";
if( "Result" is not null and "Result" = 0 ) then "Result" = null;
suspend;
end ^
/*--------------------------------------------------------------*/

create procedure "REPC_PROC_COUNT_DOCUMENTS"(in_db int, in_user varchar(32))
returns("Result_all" int, "Result_user" int)
as
begin
"Result_all" = null;
"Result_user" = null;
if( in_db is null ) then begin suspend; exit; end

if( in_user is not null ) then
  select count(*), count(case when "CreateUser" = :in_user then 1 else null end)
  from "REPC_DOCUMENTS" where "Id_Database" = :in_db into :"Result_all", :"Result_user";
else
  select count(*) from "REPC_DOCUMENTS" where "Id_Database" = :in_db into :"Result_all";

if( "Result_all" is not null and "Result_all" = 0 ) then "Result_all" = null;
if( "Result_user" is not null and "Result_user" = 0 ) then "Result_user" = null;
suspend;
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_CONFIG_DELETE" for "REPC_CONFIG"
before delete
as
begin
  if ( old."Id" = 1 ) then exception "REPC_EXC_CONFIG_1";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_PROJECTS_INSERT" for "REPC_PROJECTS"
before insert
as
begin
  if( new."Id" is null ) then new."Id" = gen_id("REPC_PROJECTS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_PROJECTS_DELETE" for "REPC_PROJECTS"
before delete
as
begin
if( exists(select "Id" from "REPC_DATABASES" where "Id_Project" = old."Id") ) then exception "REPC_EXC_PROJECTS_1";
if( exists(select "Id" from "REPC_GROUPS" where "Id_Project" = old."Id") ) then exception "REPC_EXC_PROJECTS_2";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DATABASES_INSERT" for "REPC_DATABASES"
before insert
as
begin
  if( exists( select "Id" from "REPC_DATABASES" where "Id_Project" = new."Id_Project"
  and "Name" = new."Name" ) ) then exception "REPC_EXC_DATABASES_1";

  if( new."Conn_string" is null ) then new."Conn_string" = '';
  if( new."Id" is null ) then new."Id" = gen_id("REPC_DATABASES_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DATABASES_UPDATE" for "REPC_DATABASES"
before update
as
begin
  if( new."Name" != old."Name" ) then
  begin
    if( exists( select "Id" from "REPC_DATABASES" where "Id" != old."Id" and "Id_Project" = new."Id_Project"
    and "Name" = new."Name" ) ) then exception "REPC_EXC_DATABASES_1";
  end

  if( new."Conn_string" is null ) then new."Conn_string" = '';
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DATABASES_DELETE" for "REPC_DATABASES"
before delete
as
begin
if( exists(select "Id" from "REPC_DBUSERS" where "Id_Database" = old."Id") ) then exception "REPC_EXC_DATABASES_2";
if( exists(select "Id" from "REPC_DOCUMENTS" where "Id_Database" = old."Id") ) then exception "REPC_EXC_DATABASES_3";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DBUSERS_INSERT" for "REPC_DBUSERS"
before insert
as
begin
  if( exists( select "Id" from "REPC_DBUSERS" where "Id_Database" = new."Id_Database"
  and "UserName" = new."UserName" ) ) then exception "REPC_EXC_DBUSERS_1";

  if( new."Id" is null ) then new."Id" = gen_id("REPC_DBUSERS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DBUSERS_UPDATE" for "REPC_DBUSERS"
before update
as
begin
  if( new."UserName" != old."UserName" ) then
  begin
    if( exists( select "Id" from "REPC_DBUSERS" where "Id" != old."Id" and "Id_Database" = new."Id_Database"
    and "UserName" = new."UserName" ) ) then exception "REPC_EXC_DBUSERS_1";
  end
end ^
/*--------------------------------------------------------------*/

create trigger "GROUPS_INSERT" for "REPC_GROUPS"
before insert
as
begin
  if( exists( select "Id" from "REPC_GROUPS" where "Id_Project" = new."Id_Project"
  and ("Name" = new."Name" or "Code" = new."Code") ) ) then exception "REPC_EXC_GROUPS_1";

  if( new."Id" is null ) then new."Id" = gen_id("REPC_GROUPS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_GROUPS_UPDATE" for "REPC_GROUPS"
before update
as
begin
if( new."Name" != old."Name" or new."Code" != old."Code" ) then
begin
  if( exists( select "Id" from "REPC_GROUPS" where "Id" != old."Id" and "Id_Project" = new."Id_Project"
  and ("Name" = new."Name" or "Code" = new."Code") ) ) then exception "REPC_EXC_GROUPS_1";
end
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_GROUPS_DELETE" for "REPC_GROUPS"
before delete
as
begin
if( exists(select "Id" from "REPC_REPORTS" where "Id_Group" = old."Id") ) then exception "REPC_EXC_GROUPS_2";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_REPORTS_INSERT" for "REPC_REPORTS"
before insert
as
declare variable idproj int;
begin
  idproj = null;
  select "Id_Project" from "REPC_GROUPS" where "Id" = new."Id_Group" into :idproj;

  if( idproj is not null ) then
  begin
    if( exists( select r."Id" from "REPC_REPORTS" r join "REPC_GROUPS" g on (g."Id" = r."Id_Group")
    where r."Name" = new."Name" and g."Id_Project" = :idproj ) ) then exception "REPC_EXC_REPORTS_1";
  end

  new."CreateTime" = current_timestamp;
  if( new."Id" is null ) then new."Id" = gen_id("REPC_REPORTS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_REPORTS_UPDATE" for "REPC_REPORTS"
before update
as
declare variable idproj int;
begin
  if( new."Name" != old."Name" ) then
  begin
    idproj = null;
    select "Id_Project" from "REPC_GROUPS" where "Id" = new."Id_Group" into :idproj;

    if( idproj is not null ) then
    begin
      if( exists( select r."Id" from "REPC_REPORTS" r join "REPC_GROUPS" g on (g."Id" = r."Id_Group")
      where r."Id" != old."Id" and r."Name" = new."Name" and g."Id_Project" = :idproj ) ) then exception "REPC_EXC_REPORTS_1";
    end
  end

  new."ModTime" = current_timestamp;
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_REPORT_PARAMS_INSERT" for "REPC_REPORT_PARAMS"
before insert
as
begin
  if( new."Id" is null ) then new."Id" = gen_id("REPC_REPORT_PARAMS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DOCUMENTS_INSERT" for "REPC_DOCUMENTS"
before insert
as
begin
  if( exists( select "Id" from "REPC_DOCUMENTS" where "Id_Database" = new."Id_Database"
  and "Name" = new."Name" ) ) then exception "REPC_EXC_DOCUMENTS_7";

  new."CreateUser" = user;
  new."CreateTime" = current_timestamp;
  if( new."Id" is null ) then new."Id" = gen_id("REPC_DOCUMENTS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DOCUMENTS_UPDATE" for "REPC_DOCUMENTS"
before update
as
begin
  if( user != 'REPC_ADMIN' and user != old."CreateUser" ) then exception "REPC_EXC_DOCUMENTS_8";
  if( new."Id_Database" != old."Id_Database" ) then exception "REPC_EXC_DOCUMENTS_1";
  if( old."Desc" is not null and (new."Desc" is null or new."Desc" != old."Desc") ) then exception "REPC_EXC_DOCUMENTS_2";
  if( old."Sfr" is not null and (new."Sfr" is null or new."Sfr" != old."Sfr") ) then exception "REPC_EXC_DOCUMENTS_3";
  if( old."Length" is not null and coalesce(new."Length",0) != coalesce(old."Length",0) ) then exception "REPC_EXC_DOCUMENTS_4";
  if( new."CreateUser" != old."CreateUser" ) then exception "REPC_EXC_DOCUMENTS_5";
  if( new."CreateTime" != old."CreateTime" ) then exception "REPC_EXC_DOCUMENTS_6";

  if( new."Name" != old."Name" ) then
  begin
    if( exists( select "Id" from "REPC_DOCUMENTS" where "Id_Database" = new."Id_Database"
    and "Name" = new."Name" and "Id" != old."Id" ) ) then exception "REPC_EXC_DOCUMENTS_7";
  end
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DOCUMENTS_DELETE" for "REPC_DOCUMENTS"
before delete
as
begin
  if( user != 'REPC_ADMIN' and user != old."CreateUser" ) then exception "REPC_EXC_DOCUMENTS_8";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_REPORT_GRANTS_INSERT" for "REPC_REPORT_GRANTS"
before insert
as
begin
  if( new."Id" is null ) then new."Id" = gen_id("REPC_REPORT_GRANTS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_REPORT_GRANTS_UPDATE" for "REPC_REPORT_GRANTS"
before update
as
begin
exception "REPC_EXC_REPORT_GRANTS_1";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DOCUMENT_GRANTS_INSERT" for "REPC_DOCUMENT_GRANTS"
before insert
as
declare variable cruser varchar(32);
begin
  if( user != 'REPC_ADMIN' ) then
  begin
    cruser = null;
    select "CreateUser" from "REPC_DOCUMENTS" where "Id" = new."Id_Document" into :cruser;
    if( cruser is not null and user != cruser ) then exception "REPC_EXC_DOCUMENT_GRANTS_1";
  end

  if( new."Id" is null ) then new."Id" = gen_id("REPC_DOCUMENT_GRANTS_GEN", 1);
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DOCUMENT_GRANTS_UPDATE" for "REPC_DOCUMENT_GRANTS"
before update
as
begin
exception "REPC_EXC_DOCUMENT_GRANTS_2";
end ^
/*--------------------------------------------------------------*/

create trigger "REPC_DOCUMENT_GRANTS_DELETE" for "REPC_DOCUMENT_GRANTS"
before delete
as
declare variable cruser varchar(32);
begin
  if( user != 'REPC_ADMIN' ) then
  begin
    cruser = null;
    select "CreateUser" from "REPC_DOCUMENTS" where "Id" = old."Id_Document" into :cruser;
    if( cruser is not null and user != cruser ) then exception "REPC_EXC_DOCUMENT_GRANTS_1";
  end
end ^
/*--------------------------------------------------------------*/

set term ;^
/*--------------------------------------------------------------*/

insert into "REPC_CONFIG"("Id", "Version") values(1, 1.0);
/*--------------------------------------------------------------*/

grant execute on procedure "REPC_PROC_CMP" to PUBLIC;
grant execute on procedure "REPC_PROC_COUNT_REPORTS" to PUBLIC;
grant execute on procedure "REPC_PROC_COUNT_DB" to PUBLIC;
grant execute on procedure "REPC_PROC_COUNT_DBUSERS" to PUBLIC;
grant execute on procedure "REPC_PROC_COUNT_PRUSERS" to PUBLIC;
grant execute on procedure "REPC_PROC_COUNT_DOCUMENTS" to PUBLIC;

grant select on "REPC_CONFIG" to PUBLIC;
grant select on "REPC_PROJECTS" to PUBLIC;
grant select on "REPC_DATABASES" to PUBLIC;
grant select on "REPC_DBUSERS" to PUBLIC;
grant select on "REPC_GROUPS" to PUBLIC;
grant select on "REPC_REPORTS" to PUBLIC;
grant all on "REPC_REPORT_PARAMS" to PUBLIC;
grant all on "REPC_DOCUMENTS" to PUBLIC;
grant select on "REPC_REPORT_GRANTS" to PUBLIC;
grant select, insert, delete on "REPC_DOCUMENT_GRANTS" to PUBLIC;

commit;

/* for Firebird 3 */
grant usage on generator "REPC_REPORT_PARAMS_GEN" to PUBLIC;
grant usage on generator "REPC_DOCUMENTS_GEN" to PUBLIC;
grant usage on generator "REPC_DOCUMENT_GRANTS_GEN" to PUBLIC;
grant usage on exception "REPC_EXC_CONFIG_1" to PUBLIC;
grant usage on exception "REPC_EXC_PROJECTS_1" to PUBLIC;
grant usage on exception "REPC_EXC_PROJECTS_2" to PUBLIC;
grant usage on exception "REPC_EXC_DATABASES_1" to PUBLIC;
grant usage on exception "REPC_EXC_DATABASES_2" to PUBLIC;
grant usage on exception "REPC_EXC_DATABASES_3" to PUBLIC;
grant usage on exception "REPC_EXC_DBUSERS_1" to PUBLIC;
grant usage on exception "REPC_EXC_GROUPS_1" to PUBLIC;
grant usage on exception "REPC_EXC_GROUPS_2" to PUBLIC;
grant usage on exception "REPC_EXC_REPORTS_1" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_1" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_2" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_3" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_4" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_5" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_6" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_7" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENTS_8" to PUBLIC;
grant usage on exception "REPC_EXC_REPORT_GRANTS_1" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENT_GRANTS_1" to PUBLIC;
grant usage on exception "REPC_EXC_DOCUMENT_GRANTS_2" to PUBLIC;
commit;

